@@ -4,7 +4,8 @@ from . import mesh
 from . import quadrature
 from . import doftools
 from . import element
-from . import field
+
+# from . import field
 from . import tools
 
 # from . import region
@@ -13,8 +14,8 @@ from . import solve
 from . import utils
 
 from .region import Region
-from .forms import IntegralForm, IntegralFormMixed, IntegralFormAxisymmetric
-from .field import Field, FieldAxisymmetric
+from .form import IntegralForm, IntegralFormMixed, IntegralFormAxisymmetric
+from .field import Field, FieldAxisymmetric, FieldMixed
 from .doftools import Boundary
 
 
