@@ -28,6 +28,8 @@ along with Felupe.  If not, see <http://www.gnu.org/licenses/>.
 import numpy as np
 import quadpy
 
+from types import SimpleNamespace
+
 
 class Scheme:
     def __init__(self, points, weights):
@@ -60,19 +62,65 @@ class GaussLegendre(Scheme):
 
 class Triangle(Scheme):
     def __init__(self, order: int):
-        scheme = quadpy.t2.get_good_scheme(order)
-        triangle = np.array([[0, 0], [1, 0], [0, 1]])
+        scheme = SimpleNamespace()
         area = 1 / 2
 
-        points = np.dot(triangle.T, scheme.points).T
+        if order == 1:
+            scheme.points = np.ones((1, 3)) / 3
+            scheme.weights = np.ones(1)
+
+        elif order == 2:
+            a = 2 / 3
+            b = 1 / 6
+            scheme.points = np.array([[a, b, b], [b, a, b], [b, b, a]])
+            scheme.weights = np.ones(3) / 3
+
+        elif order == 3:
+            a = 0.6
+            b = 0.2
+            c = 1 / 3
+            scheme.points = np.array([[c, c, c], [b, a, a], [a, b, a], [a, a, b]])
+            scheme.weights = np.array([-27 / 48, 25 / 48, 25 / 48, 25 / 48])
+
+        else:
+            raise NotImplementedError("order must be either 1, 2 or 3.")
+
+        triangle = np.array([[0, 0], [1, 0], [0, 1]])
+        points = np.dot(triangle.T, scheme.points.T).T
+
         super().__init__(points, scheme.weights * area)
 
 
 class Tetrahedron(Scheme):
     def __init__(self, order: int):
-        scheme = quadpy.t3.get_good_scheme(order)
-        tetra = np.array([[0, 0, 0], [1, 0, 0], [0, 1, 0], [0, 0, 1]])
-        area = 1 / 6
+        scheme = SimpleNamespace()
+        volume = 1 / 6
 
-        points = np.dot(tetra.T, scheme.points).T
-        super().__init__(points, scheme.weights * area)
+        if order == 1:
+            scheme.points = np.ones((1, 4)) / 4
+            scheme.weights = np.ones(1)
+
+        elif order == 2:
+            a = 0.58541020
+            b = 0.13819660
+            scheme.points = np.array(
+                [[a, b, b, b], [b, a, b, b], [b, b, a, b], [b, b, b, a]]
+            )
+            scheme.weights = np.ones(4) / 4
+
+        elif order == 3:
+            a = 1 / 6
+            b = 1 / 2
+            c = 1 / 4
+            scheme.points = np.array(
+                [[c, c, c, c], [b, a, a, a], [a, b, a, a], [a, a, b, a], [a, a, a, b]]
+            )
+            scheme.weights = np.array([-4 / 5, 9 / 20, 9 / 20, 9 / 20, 9 / 20])
+
+        else:
+            raise NotImplementedError("order must be either 1, 2 or 3.")
+
+        tetra = np.array([[0, 0, 0], [1, 0, 0], [0, 1, 0], [0, 0, 1]])
+
+        points = np.dot(tetra.T, scheme.points.T).T
+        super().__init__(points, scheme.weights * volume)
