@@ -39,7 +39,7 @@ from .math import det, inv
 class Region:
     """A numeric region."""
 
-    def __init__(self, mesh, element, quadrature):
+    def __init__(self, mesh, element, quadrature, grad=True):
         """A numeric region, created by a combination of a mesh,
         an element and a numeric integration scheme (quadrature).
         """
@@ -52,11 +52,6 @@ class Region:
         # ----
         # basis function "b" evaluated at quadrature point "p"
         self.h = np.array([self.element.basis(p) for p in self.quadrature.points]).T
-        
-        # hinv_bp
-        # ----
-        # inverse basis function "b" evaluated at 1 / quadrature point "p"
-        self.hinv = np.array([self.element.basis(p) for p in 1 / self.quadrature.points]).T
 
         # dhdr_bJp
         # --------
@@ -66,31 +61,31 @@ class Region:
             [self.element.basisprime(p) for p in self.quadrature.points]
         ).transpose(1, 2, 0)
 
-        if self.element.nbasis > 1:
+        if self.element.nbasis > 1 and self.mesh.ndim == self.element.ndim and grad:
 
             # dXdr_IJpe and its inverse drdX_IJpe
             # -----------------------------------
             # geometric gradient as partial derivative of undeformed coordinate "I"
             # w.r.t. natural coordinate "J" evaluated at quadrature point "p"
             # for every cell "c"
-            dXdr = np.einsum(
+            self.dXdr = np.einsum(
                 "cbI,bJp->IJpc", self.mesh.points[self.mesh.cells], self.dhdr
             )
-            drdX = inv(dXdr)
+            self.drdX = inv(self.dXdr)
 
             # dV_pe = det(dXdr)_pc * w_p
             # determinant of geometric gradient evaluated at quadrature point "p"
             # for every cell "c" multiplied by corresponding quadrature weight
             # denoted as "differential volume element"
-            self.dV = det(dXdr) * self.quadrature.weights.reshape(-1, 1)
+            self.dV = det(self.dXdr) * self.quadrature.weights.reshape(-1, 1)
 
             # dhdX_bJpc
             # ---------
             # partial derivative of basis function "b"
             # w.r.t. undeformed coordinate "J" evaluated at quadrature point "p"
             # for every cell "c"
-            self.dhdX = np.einsum("bIp,IJpc->bJpc", self.dhdr, drdX)
+            self.dhdX = np.einsum("bIp,IJpc->bJpc", self.dhdr, self.drdX)
 
     def volume(self, detF=1):
-        "Calculate cell volume for cell 'c'."
+        "Calculate cell volumes for cell 'c'."
         return np.einsum("pc->c", detF * self.dV)
