@@ -34,7 +34,7 @@ from scipy.interpolate import interp1d
 
 import meshio
 
-from .forms import IntegralFormMixed
+from .form import IntegralFormMixed
 from .solve import partition, solve
 from .math import identity, grad, dot, transpose, eigvals, det, interpolate, norms
 from .field import Field
@@ -42,6 +42,7 @@ from .field import Field
 from .mesh import revolve
 from .region import Region
 from .quadrature import GaussLegendre
+from .element import Quad, Hexahedron
 
 from .doftools import apply, partition as dofpartition, extend as dofextend
 
@@ -82,15 +83,15 @@ def newtonrhapson(
     dV = region.dV
 
     # deformation gradient at integration points
-    F = identity(grad(fields[0])) + grad(fields[0])
+    F = identity(grad(fields.fields[0])) + grad(fields.fields[0])
 
     # PK1 stress and elasticity matrix
-    f = fun_f(F, *[interpolate(f) for f in fields[1:]])
-    A = fun_A(F, *[interpolate(f) for f in fields[1:]])
+    f = fun_f(F, *[interpolate(f) for f in fields.fields[1:]])
+    A = fun_A(F, *[interpolate(f) for f in fields.fields[1:]])
 
     # assembly
     r = IntegralFormMixed(f, fields, dV).assemble(parallel=parallel).toarray()[:, 0]
-    K = IntegralFormMixed(A, fields, dV).assemble(parallel=parallel)
+    K = IntegralFormMixed(A, fields, dV, fields).assemble(parallel=parallel)
 
     converged = False
 
@@ -102,20 +103,20 @@ def newtonrhapson(
         if np.any(np.isnan(dfields[0])):
             break
         else:
-            for field, dfield in zip(fields, dfields):
+            for field, dfield in zip(fields.fields, dfields):
                 field += dfield
 
         # deformation gradient at integration points
-        F = identity(grad(fields[0])) + grad(fields[0])
+        F = identity(grad(fields.fields[0])) + grad(fields.fields[0])
 
         # PK1 stress and elasticity matrix
-        f = fun_f(F, *[interpolate(f) for f in fields[1:]])
+        f = fun_f(F, *[interpolate(f) for f in fields.fields[1:]])
 
         # residuals and stiffness matrix components
         r = IntegralFormMixed(f, fields, dV).assemble(parallel=parallel).toarray()[:, 0]
 
         rref = np.linalg.norm(r[dof0])
-        uref = np.linalg.norm(fields[0].values.ravel()[dof0])
+        uref = np.linalg.norm(fields.fields[0].values.ravel()[dof0])
         if rref == 0:
             rref = 1
         if uref == 0:
@@ -135,10 +136,10 @@ def newtonrhapson(
             break
         else:
             # elasticity matrix
-            A = fun_A(F, *[interpolate(f) for f in fields[1:]])
+            A = fun_A(F, *[interpolate(f) for f in fields.fields[1:]])
 
             # assembly of stiffness matrix components
-            K = IntegralFormMixed(A, fields, dV).assemble(parallel=parallel)
+            K = IntegralFormMixed(A, fields, dV, fields).assemble(parallel=parallel)
 
     Result = namedtuple(
         "Result", ["fields", "r", "K", "F", "f", "A", "unstack", "converged"]
@@ -173,7 +174,7 @@ def incsolve(
         bounds[boundary].value = move_t
 
         # obtain external displacements for prescribed dofs
-        u0ext = apply(fields[0], bounds, dof0)
+        u0ext = apply(fields.fields[0], bounds, dof0)
 
         Result = newtonrhapson(
             region,
@@ -198,11 +199,11 @@ def incsolve(
         else:
             # save results and go to next increment
             res.append(Result)
-            save(region, *Result, filename=filename)
+            save(region, *Result, filename=filename + ".vtk")
             # save(region, *Result, filename=filename + f"_{increment+1:d}")
             print("SAVED TO FILE")
 
-    savehistory(region, res, filename=filename)
+    savehistory(region, res, filename=filename + ".xdmf")
 
     return res
 
@@ -266,18 +267,23 @@ def save(
     unstack=None,
     converged=True,
     filename="result.vtk",
+    cell_data=None,
+    point_data=None,
 ):
 
     if unstack is not None:
+        if "mixed" in str(type(fields)):
+            fields = fields.fields
         u = fields[0]
     else:
         u = fields
 
     mesh = region.mesh
 
-    point_data = {
-        "Displacements": u.values,
-    }
+    if point_data is None:
+        point_data = {}
+
+    point_data["Displacements"] = u.values
 
     if r is not None:
         if unstack is not None:
@@ -312,19 +318,23 @@ def save(
 
     mesh = meshio.Mesh(
         points=mesh.points,
-        cells={mesh.cell_type: mesh.cells},  # [:, : mesh.edgepoints]},
+        cells=[(mesh.cell_type, mesh.cells),],  # [:, : mesh.edgepoints]},
         # Optionally provide extra data on points, cells, etc.
         point_data=point_data,
+        cell_data=cell_data,
     )
 
-    mesh.write(filename)
+    if filename is not None:
+        mesh.write(filename)
 
 
 def savehistory(region, results, filename="result_history.xdmf"):
 
     mesh = region.mesh
     points = mesh.points
-    cells = {mesh.cell_type: mesh.cells}  # [:, : mesh.edgepoints]}
+    cells = [
+        (mesh.cell_type, mesh.cells),
+    ]  # [:, : mesh.edgepoints]}
 
     with meshio.xdmf.TimeSeriesWriter(filename) as writer:
         writer.write_points_cells(points, cells)
@@ -337,6 +347,9 @@ def savehistory(region, results, filename="result_history.xdmf"):
             else:
                 reactionforces = r
 
+            if "mixed" in str(type(fields)):
+                fields = fields.fields
+
             u = fields[0]
 
             point_data = {
@@ -426,10 +439,12 @@ def axito3d(
     mesh_3d = revolve(mesh_axi, n=n, phi=phi)
     values_3d = revolve((field_axi.values, mesh_axi.cells), n=n, phi=phi)[0]
 
-    edict = {Quad1: Hex1}
+    edict = {Quad: Hexahedron}
 
     element_3d = edict[type(element_axi)]()
-    quadrature_3d = GaussLegendre(order=quadrature_axi.order, dim=3)
+
+    order = int((quadrature_axi.npoints / 2 - 1) ** 3)
+    quadrature_3d = GaussLegendre(order=order, dim=3)
     region_3d = Region(mesh_3d, element_3d, quadrature_3d)
     field_3d = Field(region_3d, dim=3, values=values_3d)
 
