@@ -27,10 +27,68 @@ along with Felupe.  If not, see <http://www.gnu.org/licenses/>.
 import pytest
 import numpy as np
 import felupe as fe
+import casadi as ca
 
 import numpy as np
 
 
+def mat_W1(F):
+    mu, bulk = 1, 5000
+
+    J = ca.det(F)
+    C = ca.transpose(F) @ F
+    Cu = J ** (-2 / 3) * C
+
+    return mu / 2 * (ca.trace(Cu) - 3) + bulk / 2 * (J - 1) ** 2
+
+
+def mat_W2(F, p):
+    mu, bulk = 1, 5000
+
+    J = ca.det(F)
+    C = ca.transpose(F) @ F
+    Cu = J ** (-2 / 3) * C
+
+    return mu / 2 * (ca.trace(Cu) - 3) + p * (J - 1) - 1 / (2 * bulk) * p ** 2
+
+
+def mat_W3(F, p, J):
+    mu, bulk = 1, 5000
+
+    detF = ca.det(F)
+    C = ca.transpose(F) @ F
+    Cu = detF ** (-2 / 3) * C
+
+    return mu / 2 * (ca.trace(Cu) - 3) + bulk / 2 * (J - 1) ** 2 + p * (detF - J)
+
+
+def mat_W2T(F, P):
+    mu, gamma = 1, 5000
+
+    P = ca.reshape(P, (3, 3))
+
+    C = ca.transpose(F) @ F
+    E = (C - ca.DM.eye(3)) / 2
+    I1 = ca.trace(E)
+    I2 = ca.trace(E @ E)
+
+    return mu * I2 + gamma * I1 ** 2 / 2 - ca.trace(ca.transpose(P) @ F)
+
+
+def mat_W3T(dxdX, P, F):
+    mu, gamma = 1, 5000
+
+    P = ca.reshape(P, (3, 3))
+    F = ca.reshape(F, (3, 3))
+
+    C = ca.transpose(F) @ F
+    E = (C - ca.DM.eye(3)) / 2
+    I1 = ca.trace(E)
+    I2 = ca.trace(E @ E)
+
+    return mu * I2 + gamma * I1 ** 2 / 2 + ca.trace(ca.transpose(P) @ (dxdX - F))
+
+
 def mat_straininvariants(invariants):
     """Calculate first (W_a) and second (W_ab) partial derivatives
     of the strain energy density function w.r.t. the invariants."""
@@ -106,21 +164,44 @@ def pre():
     p = fe.Field(r, dim=1)
     J = fe.Field(r, dim=1, values=1)
 
-    F = fe.tools.defgrad(u)
+    F = u.extract(grad=True, add_identity=True)
 
     return F, u, p, J
 
 
+def preT():
+    m = fe.mesh.Cube()
+    e = fe.element.Hexahedron()
+    q = fe.quadrature.GaussLegendre(1, 3)
+    r = fe.Region(m, e, q)
+    u = fe.Field(r, dim=3)
+    P = fe.Field(r, dim=9)
+    F = fe.Field(r, dim=9)
+
+    dxdX = u.extract(grad=True, add_identity=True)
+
+    return dxdX, u, P, F
+
+
 def test_basic():
     F, u, p, J = pre()
 
-    umat = fe.constitution.models.NeoHooke(1, 3)
+    umat = fe.constitution.NeoHooke(1, 3)
     umat.P(F), umat.A(F)
 
-    umat = fe.constitution.models.NeoHookeCompressible(1, 3)
+    umat = fe.constitution.NeoHookeCompressible(1, 3)
     umat.P(F), umat.A(F)
 
-    strain = fe.tools.strain(u)
+    umat = fe.constitution.LineChange()
+    umat.fun(F), umat.grad(F)
+
+    umat = fe.constitution.AreaChange()
+    umat.fun(F), umat.grad(F)
+
+    umat = fe.constitution.VolumeChange()
+    umat.fun(F), umat.grad(F), umat.hessian(F)
+
+    strain = u.grad(sym=True)
     umat = fe.constitution.models.LinearElastic(E=3, nu=0.3)
     umat.elasticity(strain)
     umat.stress(strain)
@@ -129,14 +210,14 @@ def test_basic():
 def test_invariants():
     F, u, p, J = pre()
 
-    umat = fe.constitution.models.NeoHooke(1, 3)
+    umat = fe.constitution.NeoHooke(1, 3)
     umat.P(F), umat.A(F)
 
-    umat = fe.constitution.models.NeoHookeCompressible(1, 3)
+    umat = fe.constitution.NeoHookeCompressible(1, 3)
     umat.P(F), umat.A(F)
 
-    strain = fe.tools.strain(u)
-    umat = fe.constitution.models.LinearElastic(E=3, nu=0.3)
+    strain = u.grad(sym=True)
+    umat = fe.constitution.LinearElastic(E=3, nu=0.3)
     umat.elasticity(strain)
     umat.stress(strain)
 
@@ -145,23 +226,7 @@ def test_invariants():
     mat_K = fe.constitution.Hydrostatic(bulk=200.0)
     mat = fe.constitution.Composite(mat_U, mat_K)
 
-    umat = fe.constitution.MaterialFrom(mat)
-    umat.P(F), umat.A(F)
-
-    mat_I = fe.constitution.df0da0.InvariantBased(mat_invariants)
-    mat_U = fe.constitution.df0da0.AsIsochoric(mat_I)
-    mat_K = fe.constitution.df0da0.Hydrostatic(bulk=200.0)
-    mat = fe.constitution.df0da0.Composite(mat_U, mat_K)
-
-    umat = fe.constitution.MaterialFrom(mat)
-    umat.P(F), umat.A(F)
-
-    mat_I = fe.constitution.df_da_.InvariantBased(mat_invariants)
-    mat_U = fe.constitution.df_da_.AsIsochoric(mat_I)
-    mat_K = fe.constitution.df_da_.Hydrostatic(bulk=200.0)
-    mat = fe.constitution.df_da_.Composite(mat_U, mat_K)
-
-    umat = fe.constitution.MaterialFrom(mat)
+    umat = fe.constitution.Material(mat)
     umat.P(F), umat.A(F)
 
 
@@ -173,37 +238,20 @@ def test_stretch():
     mat_K = fe.constitution.Hydrostatic(bulk=200.0)
     mat = fe.constitution.Composite(mat_U, mat_K)
 
-    umat = fe.constitution.MaterialFrom(mat)
-    umat.P(F), umat.A(F)
-
-    mat_S = fe.constitution.df0da0.PrincipalStretchBased(mat_invariants)
-    mat_U = fe.constitution.df0da0.AsIsochoric(mat_S)
-    mat_K = fe.constitution.df0da0.Hydrostatic(bulk=200.0)
-    mat = fe.constitution.df0da0.Composite(mat_U, mat_K)
-
-    umat = fe.constitution.MaterialFrom(mat, parallel=True)
-    umat = fe.constitution.MaterialFrom(mat, parallel=False)
-    umat.P(F), umat.A(F)
-
-    mat_S = fe.constitution.df_da_.PrincipalStretchBased(mat_invariants)
-    mat_U = fe.constitution.df_da_.AsIsochoric(mat_S)
-    mat_K = fe.constitution.df_da_.Hydrostatic(bulk=200.0)
-    mat = fe.constitution.df_da_.Composite(mat_U, mat_K)
-
-    umat = fe.constitution.MaterialFrom(mat, parallel=True)
-    umat = fe.constitution.MaterialFrom(mat, parallel=False)
+    umat = fe.constitution.Material(mat)
     umat.P(F), umat.A(F)
 
 
 def test_threefield():
     F, u, p, J = pre()
 
-    umat = fe.constitution.models.NeoHooke(1, 3)
+    nh = fe.constitution.models.NeoHooke(1, 3)
 
-    F, p, J = fe.tools.FpJ((u, p, J))
+    fields = fe.FieldMixed((u, p, J))
+    F, p, J = fields.extract()
 
-    vmat = fe.constitution.variation.upJ(umat.P, umat.A)
-    vmat.f(F, p, J), vmat.A(F, p, J)
+    umat = fe.constitution.GeneralizedThreeField(nh.P, nh.A)
+    umat.f(F, p, J), umat.A(F, p, J)
 
 
 def test_straininvariant():
@@ -214,24 +262,35 @@ def test_straininvariant():
     mat_K = fe.constitution.Hydrostatic(bulk=200.0)
     mat = fe.constitution.Composite(mat_U, mat_K)
 
-    umat = fe.constitution.MaterialFrom(mat)
+    umat = fe.constitution.Material(mat)
     umat.P(F), umat.A(F)
 
-    mat_E = fe.constitution.df0da0.StrainInvariantBased(mat_straininvariants)
-    mat_U = fe.constitution.df0da0.AsIsochoric(mat_E)
-    mat_K = fe.constitution.df0da0.Hydrostatic(bulk=200.0)
-    mat = fe.constitution.df0da0.Composite(mat_U, mat_K)
 
-    umat = fe.constitution.MaterialFrom(mat)
-    umat.P(F), umat.A(F)
+def test_ad():
+    F, u, p, J = pre()
 
-    mat_E = fe.constitution.df_da_.StrainInvariantBased(mat_straininvariants)
-    mat_U = fe.constitution.df_da_.AsIsochoric(mat_E)
-    mat_K = fe.constitution.df_da_.Hydrostatic(bulk=200.0)
-    mat = fe.constitution.df_da_.Composite(mat_U, mat_K)
+    fields = fe.FieldMixed((u, p, J))
+    F, p, J = fields.extract()
 
-    umat = fe.constitution.MaterialFrom(mat)
-    umat.P(F), umat.A(F)
+    umat = fe.constitution.StrainEnergyDensity(mat_W1)
+    umat.f(F), umat.A(F)
+
+    umat = fe.constitution.StrainEnergyDensityTwoField(mat_W2)
+    umat.f(F, p), umat.A(F, p)
+
+    umat = fe.constitution.StrainEnergyDensityThreeField(mat_W3)
+    umat.f(F, p, J), umat.A(F, p, J)
+
+    dxdX, u, P, F = preT()
+
+    fields = fe.FieldMixed((u, P, F))
+    dxdX, P, F = fields.extract()
+
+    umat = fe.constitution.StrainEnergyDensityTwoFieldTensor(mat_W2T)
+    umat.f(dxdX, P), umat.A(dxdX, P)
+
+    umat = fe.constitution.StrainEnergyDensityThreeFieldTensor(mat_W3T)
+    umat.f(dxdX, P, F), umat.A(dxdX, P, F)
 
 
 if __name__ == "__main__":  # pragma: no cover
@@ -240,3 +299,4 @@ if __name__ == "__main__":  # pragma: no cover
     test_straininvariant()
     test_stretch()
     test_threefield()
+    test_ad()
